module OrientDb
  class LearningObjectRepository < Base
    include OrientDb::Methods::EdgeMethods
    include RepositoriesProxy

    ##
    # Call this methods for increment the views number of a +learning_object+
    # You are creating the 'views' relation between an +user+ and +learning_object+
    # This relation is created only once.
    #
    # Nothing is returned
    def increment_views(user, learning_object)
      if !edge_exists? "Views", user.rid, learning_object.id
        create_edge "Views", user.rid, learning_object.id
        learning_object.views = learning_object.views + 1
      end
    end

    ##
    # Call this method checks if the +user+ has a like relation with +learning_object+
    # This methods returns a boolean
    def liked?(user, learning_object)
      edge_exists? 'Likes', user.rid, learning_object.id
    end

    ##
    # Call this method to create a 'like' relation between an user and a learning object.
    # If the user already has the like relation, then nothing happens.
    #
    # If you want raise an exeception, use the LearningObjectRepository::like! method.
    #
    # Usage:
    #   repository.for(:learning_objects).like current_user, @learning_object
    def like(user, learning_object)
      like_edge_class = 'Likes'

      if !liked?
        create_edge like_edge_class, user.rid, learning_object.id
        learning_object.likes = learning_object.likes + 1
      end
    end

    ##
    # Call this method to create a 'like' relation between an user and a learning object.
    # If the +user+ already liked the +learning_object+ throw an exception.
    def like!(user, learning_object)
      if liked? user, learning_object
        raise 'The user already likes this object.'
      end
      like user, learning_object
    end

    ##
    # This methods destroy a 'like' relation between +user+ and +learning_object+
    # If the +user+ hasn't liked the +learning_object+ nothing happens
    # If you want raise an exeception, use the LearningObjectRepository::dislike! method.
    def dislike(user, learning_object)
      like_edge_class = 'Likes'
      if liked? user, learning_object
        destroy_edge like_edge_class, user.rid, learning_object.id
        learning_object.likes = learning_object.likes - 1
        true
      end
    end

    ##
    # This methods destroy a 'like' relation between +user+ and +learning_object+
    # If the +user+ hasn't liked the +learning_object+ throw an exception.
    def dislike!(user, learning_object)
      if liked? user, learning_object
        dislike user, learning_object
      end
      raise 'The user hasn`t likes this object.'
    end

    def get_by_dspace_id(id_dspace)
      result = select_by_property(odb_class, "id_dspace", id_dspace)
      build_object result.first
    end

    ##
    # This method return all learning objects of +user+ scope.
    #
    # No users have objects.
    # Learning Object submission was not implemented yet.
    # For now, an empty array is returned.
    def all(user)
      #result = select_by_property(odb_class, "p_id", user.id)
      #build_objects result
      []
    end

    def get_subjects(learning_object)
      result = get_edges_end("IsAbout", "out", learning_object.id)
      subject_repository.build_objects(result)
    end

    def get_attributes(learning_object)
      result = get_edges_end("HasAttr", "out", learning_object.id)
      attribute_repository.build_objects(result)
    end

    def create_relations(learning_object)
      edges = []
      unless learning_object.subjects.empty?
        edges << create_edges_from_array("IsAbout", learning_object.id, learning_object.subjects, true)
      end
      unless learning_object.attributes.empty?
        edges << create_edges_from_array("HasAttr", learning_object.id, learning_object.attributes, true)
      end
      edges.flatten
    end

    def save(learning_object)
      if learning_object.valid?
        connection.command "INSERT INTO LearningObject CONTENT #{learning_object.to_json}"
      end
    end

    def update_property(learning_object, property, value)
      if accepted_properties.include? property
        connection.command "UPDATE LearningObject SET #{property}='#{value}' WHERE @rid = #{learning_object.id}"
      end
    end

    ##
    # To create "index:learningobject_search" on OrientDB, use the following command:
    #   CREATE INDEX learningobject_search
    #   ON LearningObject (name, description)
    #   FULLTEXT ENGINE LUCENE
    #   METADATA {"analyzer":"org.apache.lucene.analysis.br.BrazilianAnalyzer"}
    def search(params)
      # TODO: fix search metadata from dspace
      order = params[:order]
      qry = params[:query]
      unless order.nil?
        order = case order
        when "author"
          "author"
        when "publicationasc"
          "created_at"
        when "publicationdesc"
          "created_at DESC"
        when "title"
          "name ASC"
        else
          nil
        end
      end

      query = "SELECT EXPAND(rid) FROM index:learningobject_search WHERE key LUCENE '#{qry}'"
      query = "SELECT * FROM (" + query + ") WHERE " + fetch_types(params[:type]) unless params[:type].blank?
      query = "SELECT * FROM (" + query + ") WHERE " + fetch_year(params[:year]) unless params[:year].blank?
      query = "SELECT * FROM (" + query + ") ORDER BY #{order}" unless order.nil?
      query = "SELECT @rid.asString(), last_modified FROM (" + query + ")"
      learning_objects_hash = connection.query query, limit: 10000
      # return only rids with their modification time
      learning_objects_hash.map do |e|
        {'@rid': e['rid'], 'last_modified': e['last_modified']}
      end
    end

    def build_object(args={})
      LearningObjectBuilder.build_from_orientdb args
    end

    def build_hash(object)
      hash = super(object)
      hash.delete("likes")
      hash.delete("views")
      hash.delete("downloads")
      hash.delete("subjects")
      hash.delete("attributes")
      hash
    end

    def fetch_year(years)
      if years.size() == 1
        year = years[0]
        qry = "created_at.format('yyyy') = '#{year}'"
      elsif years.size() == 2
        begin_year = years[0]
        end_year = years[1]
        qry = "created_at.format('yyyy') >= '#{begin_year}' AND created_at.format('yyyy') <= '#{end_year}'"
      end
    end

    def fetch_types(types)
      qry = ''
      types.each do |type|
        qry = qry + "type = '#{type}' " if type == types.first
        qry = qry + "OR type = '#{type}' " if type != types.first
      end
      qry
    end

    private

    def accepted_properties
      ['thumbnail']
    end

    def odb_class
      "LearningObject"
    end

    def count_likes(learning_object)
      get_in_edges_count "Likes", learning_object.id
    end

    def count_views(learning_object)
      get_in_edges_count "Views", learning_object.id
    end

  end
end
